/******  EHX B9 upgrade (P. GIORGIO 2020 - Covid-19 time)
          - Oled Display
           - Rotary Encoder
             - Preset Function
               - Leslie speed-up function
                 - Mute/DRY function
                                *****/

#include <SPI.h>                   // OLED library
#include <Wire.h>                  // OLED
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
// #include <Fonts/FreeSans24pt7b.h>
#include <Fonts/FreeSans18pt7b.h>

#define SCREEN_WIDTH 128           // OLED display width, in pixels
#define SCREEN_HEIGHT 64           // OLED display height, in pixels

      // Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

// 1.  Fat & Full- This sound adds an extra octave below and above to make your guitar sound twice as big. Fills out any band in an instant! MOD Type: Chorus.
// 2.  Jazz- This preset has the cool, smooth jazz tone reminiscent of the late great organist Jimmy Smith. MOD Type: Chorus.
// 3.  Gospel- This preset has the upper octave drawbars added to capture that great soulful organ tone. MOD Type: Chorus.
// 4.  Classic Rock- This preset captures the classic rock sound of songs like Procol Harum’s “Whiter Shade of Pale.” Add a touch of distortion for a classic dirty organ. MOD Type: Chorus.
// 5.  Bottom End- This preset has the lower draw bar sound. Perfect for adding bottom to your guitar or playing B3 bass sounds. With the CLICK control up you can lay down a bass line like the one on Sugarloaf’s “Green Eyed Lady.” MOD Type: Chorus.
// 6.  Octaves- This preset uses the fundamental tone plus one octave above. This sound is great for songs like Led Zeppelin’s “Your Time is Going to Come.” MOD Type: Chorus. CLICK control adds not only key click, but higher harmonics.
// 7.  Cathedral- Turn up the reverb and you are at the seat of a giant cathedral organ! Psychedelic rock tones easily pour out. MOD Type: Tremolo. CLICK adjusts the tremolo depth.
// 8.  Continental- This is the classic combo organ sound similar to classic songs “96 Tears,” “Woolly Bully” and “House of the Rising Sun.” MOD Type: Vibrato. CLICK controls vibrato depth.
// 9.  Bell Organ- If you crossed an electric piano with an organ this is it. MOD Type:  Tremolo. CLICK adjusts the amount of bell or chime added to the sound.

// MULTIPLEXER Position 0 -> command back to the B9 rotaty switch: this function could be use to recall the PRESET on the Rotary Switch
// MULTIPLEXER Position 1 -> FAT & FULL
// MULTIPLEXER Position 2 -> JAZZ
// MULTIPLEXER Position 3 -> GOSPEL
// MULTIPLEXER Position 4 -> CLASSIC ROCK
// MULTIPLEXER Position 5 -> BOTTOM END
// MULTIPLEXER Position 6 -> OCTAVES
// MULTIPLEXER Position 7 -> CATHEDRAL
// MULTIPLEXER Position 8 -> CONTINENTAL
// MULTIPLEXER Position 9 -> RHODES

char *textTop[] = {"electro-harmonix  NYC" , "1 octave below & above" , "2    Jimmy Smith     " , " 3  upper octave   ", "4   Procol Harum    ", "5   Deep Purple   " , "6   Led Zeppelin   " , "7    CATHEDRAL       " , "8 House of the Rising" , "9      Bell      " };
char *textMid[] = {"      GP      MOD    " , "     Leslie   MOD    " , "     Leslie   MOD    " , "     Leslie   MOD    " , "     Leslie   MOD    " ,"     Leslie   MOD    " ,"     Leslie   MOD    " , "     Tremolo  MOD    " , "     Vibrato  MOD    " , "     Tremolo  MOD    " };
char *textBottom[] = {"     B9  ", "Fat+Full", "  JAZZ ", " Gospel", " ROCK ", " Bottom" , "Octaves" , "JS Bach" , " Farfisa" , "Rhodes" };

int tTop = 0;
int tMid = 0;
int tBottom = 10;
int del = 25; 

float pressLength_milliSeconds = 0;  // to define how long the Foot Switch has been pressed
int optionOne_milliSeconds = 100;    // short press -> ON/OFF function
int optionTwo_milliSeconds = 450;    // long press  -> Leslie Speed-up
int onOff = 0;                       // flag if B9 is OFF -> 0     ON -> 1 
int latchSim = 0;                    // flag to simulate latching of the foot switch
int dry = HIGH;                      // flag to keep the DRY signal passing through (keep footSwitchDry HIGH while B9 is OFF)
int fadeAmount = 3;                  // how many points to fade the MOD by
int velModLeslie = 0;                // leslie MOD speed (position of the Potenziometer MOD)
int Mod = 0;

const int CS_PIN = 10;               // set pin 10 as the slave select (SS) for the digital Potenziometer MCP42100
int potValueMod = 255;               // value of the digital Potenziometer MCP42100 (255 --> 3.3V. --> min MOD) 

/***********************MCP42XXX Commands************************/
//potentiometer select byte
const int POT0_SEL = 0x11;      // command to the digital potenziometer 0 pin(8-9-10)
const int POT1_SEL = 0x12;      // command to the digital potenziometer 1 pin(5-6-7)
const int BOTH_POT_SEL = 0x13;  // command to the digital potenziometers 0+1 pin(5-6-7)+(8-9-10)

/********************************74HC4067 Commands*********************************/
// connect 74HC4067 S0~S3 to Arduino EVERY NANO A3~A0 respectively (PD0-PD1-PD2-PD3)

byte controlPins[] = {B00000000,
                      B00000001,
                      B00000010,
                      B00000011,
                      B00000100,
                      B00000101,
                      B00000110,
                      B00000111,
                      B00001000,
                      B00001001
                     };
/********************************74HC4067 Commands*********************************/

static int pinA = 2;            // First hardware interrupt pin is digital pin 2  -> Encoder signal A   ->
static int pinB = 3;            // Second hardware interrupt pin is digital pin 3 -> Encoder signal B   ->
static int encoderButton = 4;   // Encoder Push Button D4                                               ->  for all the INPUT the COMMON is on the GND
static int footSwitch = 5;      // Momentary Foot Switch Button                                         -> 
const int footSwitchOFF = 6;    // foot switch Position OFF ->  Output D6 HIGH for connection 3-4 when B9 is OFF
const int footSwitchDRY = 7;    // foot switch Position OFF ->  Output D7 HIGH for connection 2-6 when B9 is OFF -> DRY signal pass (if OFF -> MUTE the DRY signal)
const int footSwitchON = 8;     // foot switch Position ON  ->  Output D8 HIGH for connection 5-6 when B9 is ON


const int potPinMod = A7;       // input from Potenziometer "MOD" of B9 (connections:    MAX -> GND     min -> 5V.    Wiper -> A7  )
volatile byte encoderPos = 0;
volatile byte pinALast = HIGH;
volatile byte stateA = HIGH;

int encoderButtonState = 0;           // read the encoderButton
int previousEncoderButtonState = 0;
int rotaryEnable = 0;                 // flag to give back the command to the mechanical Rotary Switch on the B9
   
void setup() {
  pinMode(encoderButton, INPUT_PULLUP); // set Encoder Push Button
  pinMode(pinA, INPUT_PULLUP);          // set pinA encoder CHANNEL A
  pinMode(pinB, INPUT_PULLUP);          // set pinB encoder CHANNEL B 
  pinMode(footSwitch, INPUT_PULLUP);    // set footSwitch
  pinMode(footSwitchDRY, OUTPUT);       // Contact A -> HIGH if footSwitch OFF (2-6) 
     digitalWrite(footSwitchDRY,HIGH); 
  pinMode(footSwitchOFF, OUTPUT);       // Contact B -> HIGH if footSwitch OFF (3-4)
     digitalWrite(footSwitchOFF,HIGH);
  pinMode(footSwitchON, OUTPUT);        // Contact C -> HIGH if footSwitch ON (5-6)
     digitalWrite(footSwitchON,LOW);
 
  pinMode(CS_PIN, OUTPUT);     // set the CS_PIN as an output:
  SPI.begin();     // initialize SPI:

    VPORTD.DIR = B00001111;               // set PORTD A0-A1-A2-A3 (PD3-PD2-PD1-PD0) to outputs - 74HC4067 Multiplexer
  
attachInterrupt(digitalPinToInterrupt(pinB),PinB,RISING); // set an interrupt on PinB, looking for a RISING edge signal and executing the "PinB" Interrupt Service Routine (below)  
attachInterrupt(digitalPinToInterrupt(pinA),PinA,FALLING); // set an interrupt on PinA, looking for a FALLING edge signal and executing the "PinA" Interrupt Service Routine (below)

 // Serial.begin (9600); // start the serial monitor link
     
           // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address 0x3C for 128x64
          //   Serial.println(F("SSD1306 allocation failed"));
   for(;;);      // Don't proceed, loop forever
                                                 }
            }

      void PinA(){
        cli(); //stop interrupts happening before we read pin values
        stateA = digitalRead(pinA);
 
       if ((pinALast == HIGH) && (stateA == LOW)) {
          if (digitalRead(pinB) == HIGH) {  
             if(encoderPos == 0) {encoderPos = 10;}
      encoderPos--;
           } else {
                    if(encoderPos == 9) {encoderPos = 0;}
      encoderPos++;
                   }
                                                  }
       pinALast = stateA;
       sei(); //restart interrupts
            }

       void PinB(){
         cli(); //stop interrupts happening before we read pin values
         stateA = digitalRead(pinA);
  
          if ((pinALast == HIGH) && (stateA == LOW)) {
             if (digitalRead(pinB) == HIGH) {  
                if(encoderPos == 0) {encoderPos = 10;}
        encoderPos--;
              } else {
                       if(encoderPos == 9) {encoderPos = 0;}
        encoderPos++;
           }
                                             }
        pinALast = stateA;
        sei(); //restart interrupts
            }

void setPin(int outputPin){              // function to select Ports on Nano and pin on 74HC4067  MULTIPLEXER                                
  VPORTD.OUT = controlPins[outputPin];
                           }

void text(void) {
  display.clearDisplay();
  display.stopscroll();
  display.setTextSize(1);             
  display.setTextColor(SSD1306_WHITE);        // Draw white text
  display.setFont();
  display.setCursor(0,0);             // Start at top-left corner
  display.println(textTop[tTop]);

  display.setTextColor(SSD1306_BLACK, SSD1306_WHITE); // Draw 'inverse' text
  display.setCursor(0,9);             // Start at top-left corner 0,-9
  display.println(textMid[tMid]);
  display.setCursor(109,9);
  Mod = (255-velModLeslie)/2.55;
  display.println(Mod);
  display.setCursor(1,9);

  if(dry == 0) {display.println("Mute");}
  else {display.println("Dry");}
  
  display.setTextSize(1); 
  if (onOff == 0) { display.setTextSize(7);
                    display.setTextColor(SSD1306_BLACK, SSD1306_WHITE);
                    display.setCursor(0,20);
                    display.println("    ");
                    display.setTextSize(1);    
                    display.setFont(&FreeSans18pt7b);
                    display.setCursor(0,55);
                    display.println(textBottom[tBottom]);   }  // Draw 'inverse' text when B9 is OFF
  
  if (onOff == 1) { display.setTextColor(SSD1306_WHITE);       
  display.setFont(&FreeSans18pt7b);
  display.setCursor(0,55);
  display.println(textBottom[tBottom]);  }
  display.display();
  delay(del);
                }

void scrolltext(void) {
  
  display.clearDisplay();
  
  display.setTextSize(1);
  display.setTextColor(SSD1306_WHITE);
  display.setFont(&FreeSans18pt7b);
  display.setCursor(0, 55);
  display.println(textBottom[tBottom]);
  
  display.display();    
  delay(del);
  display.startscrollleft(0x00, 0x0F);                     // scroll text when Foot Switch has beep pressed longer to show that Leslie is speeding-up!
                       }

// Function Declaration
//send the command and the wiper value through SPI

void DigitalPotWrite(int cmd, int val)
{
  // constrain input value within 0 - 255
  val = constrain(val, 0, 255);
  // set the CS pin to low to select the chip:
  digitalWrite(CS_PIN, LOW);
  // send the command and value via SPI:
  SPI.transfer(cmd);
  SPI.transfer(val);
  // Set the CS pin high to execute the command:
  digitalWrite(CS_PIN, HIGH);
}


void loop(){
  
                stateA = digitalRead(pinA);
                  if ((pinALast == HIGH) && (stateA == LOW)) {
                    if (digitalRead(pinB) == HIGH) { // delay(1);
                        if(encoderPos == 0) {encoderPos = 10;}
                encoderPos--;
                    } else {
                        if(encoderPos == 9) {encoderPos = 0;}
                encoderPos++;
                           }
                                             }
                pinALast = stateA;

// the previous part is the same as the interrupt for reading the encoder: I added again because, for some misterious reason..., this improve the reliability of reading!

if(encoderPos != 0)  {
encoderButtonState = digitalRead(encoderButton);
delay(1);
if(encoderButtonState != previousEncoderButtonState) {
  if(encoderButtonState == LOW) {rotaryEnable = !rotaryEnable;}
                                                     }
                     }
 
 velModLeslie = analogRead(potPinMod)/4;    // read the position of Potenziometer MOD

  while (digitalRead(footSwitch) == LOW ){ 
    delay(100);   

     pressLength_milliSeconds = pressLength_milliSeconds + 100;  

     if (onOff == 0) {velModLeslie = 0;}   // don't speed-up if B9 is OFF
     
      if (pressLength_milliSeconds >= optionTwo_milliSeconds){                        // speed-up Leslie while hold footSwitch ON (Max Speed --> 0V.)
         for (potValueMod = velModLeslie; potValueMod > 0; potValueMod = potValueMod - fadeAmount) {
           DigitalPotWrite(BOTH_POT_SEL , potValueMod-1);
         
          if(potValueMod == velModLeslie) {scrolltext(); }
                 delay(25);    // slope to increase the speed   
                                                                                                    } 
          velModLeslie = 0;
                                                            }                                      
                                          
                                          
                                          }

  //Option 2 --> speed-down Leslie when footSwitch released (min Speed --> 3.3V.)
   
   velModLeslie = analogRead(potPinMod)/4;    // read the position of Potenziometer MOD

     
        
        if (pressLength_milliSeconds >= optionTwo_milliSeconds){
           for (potValueMod = 0; potValueMod <= velModLeslie; potValueMod = potValueMod + fadeAmount) {
             if (onOff == 0) {potValueMod = velModLeslie;}    // don't speed-down if B9 is OFF
               DigitalPotWrite(BOTH_POT_SEL , potValueMod);
                delay(25);    // slope to decrease the speed   
                                                                                                       } 
         // velModLeslie = 255;     
                                                               }

      //Option 1 - Execute the first option if the button is held for less than half second -> simulate latching footswitch
 
        else if(pressLength_milliSeconds >= optionOne_milliSeconds){
          if(latchSim == 0)       {
              onOff = 1;                       // B9 is ON
              digitalWrite(footSwitchOFF,LOW);
              digitalWrite(footSwitchDRY,LOW);
              digitalWrite(footSwitchON,HIGH);
              latchSim = 1;       }
  
           else               { 
              onOff = 0;                        // B9 is OFF
              digitalWrite(footSwitchOFF,HIGH);
              digitalWrite(footSwitchDRY,dry);
              digitalWrite(footSwitchON,LOW);
              latchSim = 0;   }
                                                                   }
  
          pressLength_milliSeconds = 0;
 
DigitalPotWrite(BOTH_POT_SEL , velModLeslie);   

 if(onOff == 0  && encoderPos == 9 && digitalRead(encoderButton) == LOW) {   // To move from DRY (instrument dry signal passing through the Organ Output) to MUTE (Mute the instrument signal on the Organ Output):
                       dry = LOW;                                            // from B9 powered but OFF, go to preset 9 (Rhodes) then press the rotary encoder once.
                       digitalWrite(footSwitchDRY,dry);                      // To go back to DRY: remove the power and start again!                                  
                       }

    if(rotaryEnable == 1) {
                       setPin(0);
                       tTop = encoderPos;
                       tMid = encoderPos;
                       tBottom = 0;
                       text();                                             
                       }
         else {
                  
                switch (encoderPos) {
                                    
                       case 0:
                       setPin(encoderPos);
                       tTop = encoderPos;
                       tMid = encoderPos;
                       tBottom = encoderPos;
                       text();
                       delay(del); 
                       
                       break;  
                       
                       case 1:  
                       setPin(encoderPos);
                       tTop = encoderPos;
                       tMid = encoderPos;
                       tBottom = encoderPos;
                       text();
                       break;
                       
                       case 2:
                       setPin(encoderPos);
                       tTop = encoderPos;
                       tMid = encoderPos;
                       tBottom = encoderPos;
                       text();
                       break; 
                        
                       case 3:  
                       setPin(encoderPos);
                       tTop = encoderPos;
                       tMid = encoderPos;
                       tBottom = encoderPos;
                       text();
                       break;
                       
                       case 4: 
                       setPin(encoderPos);
                       tTop = encoderPos;
                       tMid = encoderPos;
                       tBottom = encoderPos;
                       text();
                       break;
                         
                       case 5:  
                       setPin(encoderPos);
                       tTop = encoderPos;
                       tMid = encoderPos;
                       tBottom = encoderPos;
                       text();
                       break;
                       
                       case 6:
                       setPin(encoderPos);
                       tTop = encoderPos;
                       tMid = encoderPos;
                       tBottom = encoderPos;
                       text();
                       break; 
                       
                       case 7:
                       setPin(encoderPos);
                       tTop = encoderPos;
                       tMid = encoderPos;
                       tBottom = encoderPos;
                       text();
                       break; 
                        
                       case 8:  
                       setPin(encoderPos);
                       tTop = encoderPos;
                       tMid = encoderPos;
                       tBottom = encoderPos;
                       text();
                       break;
                       
                       case 9:
                       setPin(encoderPos);
                       tTop = encoderPos;
                       tMid = encoderPos;
                       tBottom = encoderPos;
                       text();
                       break; 
                           }
         }
   previousEncoderButtonState = encoderButtonState;      
}        
